#include "../gametoy.h"

// astroshoot game, v. 0.1

#include "astroshoot.h"

char xpos = 10, ypos = 28;
signed char xi = 0, yi = 0;
signed char xsum = 0, ysum = 0;
unsigned short lasttick = 0;
unsigned char physicstick = 0;
signed short score = 0;
signed short hiscore = 0;
char nextbullet = 0;
char firedelay = 0;

signed char monsterx[16];
signed char monstery[16];
signed char monsterhits[16];

unsigned char bulletx[32];
unsigned char bullety[32];

unsigned char seed1 = 0xa6, seed2=0xe9;

char sprite_hit(char x1, char y1, char x2, char y2)
{
	if (x1 + 8 < x2 || // right side of 1 is on the left side of 2
		x2 + 8 < x1 || // right side of 2 is on the left side of 1
		y1 + 8 < y2 || 
		y2 + 8 < y1)
		return 0;
	return 1;
}

char pixel_hit(char x1, char y1, char x2, char y2)
{
	if (x1 + 8 < x2 || 
		x2     < x1 || 
		y1 + 8 < y2 ||
		y2     < y1)
		return 0;
	return 1;
}

void reset()
{
	unsigned char i;
	firedelay = 0;
	nextbullet = 0;
	xpos = 10;
	ypos = 28;
	xi = 0;
	yi = 0;
	xsum = 0;
	ysum = 0;
	lasttick = tick - 50;
	physicstick = 0;
	score = 0;
	for (i = 0; i < 16; i++)
	{
		monsterx[i] = -32;
		bulletx[i] = 128;
		monsterhits[i] = 0;
	}

	seed1 = 0xa6;
	seed2 = 0xe9;

}

unsigned char randi()
{
	unsigned char r0 = seed1;
	r0 = (r0 << 3) | (r0 >> 5);
	seed2 ^= tick >> 12;
	r0 ^= seed2;
	seed1 = seed2;
	seed2 = r0;
	return r0;
}

void physics()
{
	unsigned char i, j;
	physicstick++;
	if (firedelay)
		firedelay--;

	if (KEY_LEFT)
	{
		if (xi > -50)
			xi-=8;
	}
	if (KEY_RIGHT)
	{
		if (xi < 50)
			xi+=8;
	}
	if (KEY_UP)
	{
		if (yi > -50)
			yi-=8;
	}
	if (KEY_DOWN)
	{
		if (yi < 50)
			yi+=8;
	}
	if (KEY_A)
	{
		if (firedelay == 0)
		{
			i = nextbullet;
			nextbullet++;
			nextbullet &= 31;
			if (bulletx[i] >= 128)
			{
				bulletx[i] = xpos + 8;
				bullety[i] = ypos + 3 + (i & 1)*3;
				firedelay = 6;
			}
		}
	}

	if (xi)
	{
		if (xi > 0) xi--; else xi++;
	}
	if (yi)
	{
		if (yi > 0) yi--; else yi++;
	}

	xsum += xi;
	ysum += yi;

	while (xsum < -60)
	{
		xsum += 60;
		xpos--;
		if (xpos < 0)
			xpos = 0;
	}

	while (xsum > 60)
	{
		xsum -= 60;
		xpos++;
		if (xpos > 120)
			xpos = 120;
	}

	while (ysum < -60)
	{
		ysum += 60;
		ypos--;
		if (ypos < 0)
			ypos = 0;
	}

	while (ysum > 60)
	{
		ysum -= 60;
		ypos++;
		if (ypos > 48)
			ypos = 48;
	}

	for (i = 0; i < 16; i++)
	{
		if (monsterx[i] > -7)
		{
			monsterx[i]--;
			if (monsterx[i] == -7)
			{
				score--;
				if (score < 0)
					score++;
			}
			if (pixel_hit(monsterx[i],monstery[i],xpos+4,ypos+4))
			{
				// die
				if (score > hiscore)
					hiscore = score;
				reset();
			}			
		}

		if (bulletx[i] < 128)
			bulletx[i]+=2;
	}

	i = (physicstick >> 3) & 15;
	if (monsterx[i] <= -7)
	{
		monsterx[i] = 127;
		monstery[i] = (randi() % 48);
		monsterhits[i] = 2;
	}

	for (i = 0; i < 16; i++)
	{
		if (monsterx[i] > 8)
		{
			for (j = 0; j < 32; j++)
			{
				if (bulletx[j] < 128)
				{
					if (pixel_hit(monsterx[i],monstery[i],bulletx[j],bullety[j]))
					{
						monsterhits[i]--;
						bulletx[j] = 128;
						if (monsterhits[i] < 0)
						{
							monsterx[i] = -32;
							score+=10;
							if (score < 0)
								score -= 10;
						}
					}
				}
			}
		}
	}
}

void mainloop()
{
	unsigned char i, j;
	unsigned short p = 0;
	unsigned char scroll = (tick >> 8) & 7;
	char temp[8];

	if ((unsigned short)(tick - lasttick) > 25)
	{
		physics();

		lasttick += 25;
	}

	if (physicstick & 1)
	{
		for (i = 0; i < 64; i++)
		{
			unsigned char c = ROMDATA(astroshoot[(i + scroll) & 7]);

			for (j = 0; j < 16; j++, p++)
			{
				fb[p] = c;
			}
		}
	}
	else
	{
		memset(fb,0,1024);
	}

	drawsprite(xpos,ypos,astroshoot+16,astroshoot+8);

	for (i = 0; i < 16; i++)
	{
		drawsprite(monsterx[i],monstery[i],astroshoot+32,astroshoot+24);
	}

	for (i = 0; i < 32; i++)
	{
		putpixel(bulletx[i], bullety[i], 1);
		//putpixel(bulletx[i]+1, bullety[i], 1);
		putpixel(bulletx[i]-1, bullety[i], 1);
		putpixel(bulletx[i]-3, bullety[i], 1);
	}

	// *definitely* needs work
	temp[0] = ((score / 10000) % 10) + '0';
	temp[1] = ((score / 1000) % 10) + '0';
	temp[2] = ((score / 100) % 10) + '0';
	temp[3] = ((score / 10) % 10) + '0';
	temp[4] = ((score) % 10) + '0';
	temp[5] = 0;
	drawtext8(0,7,temp);

	temp[0] = ((hiscore / 10000) % 10) + '0';
	temp[1] = ((hiscore / 1000) % 10) + '0';
	temp[2] = ((hiscore / 100) % 10) + '0';
	temp[3] = ((hiscore / 10) % 10) + '0';
	temp[4] = ((hiscore) % 10) + '0';
	temp[5] = 0;
	drawtext8(88,7,temp);

}
