#ifdef __cplusplus
extern "C" {
#endif

#ifdef _MSC_VER
//////////////////////////////////////////////////////////////////////
// Desktop build
//////////////////////////////////////////////////////////////////////
#include <stdlib.h>
#include <string.h>
#define PROGMEM
#define ROMDATA(x) x
#define ROMDATA16(x) x
extern int keyleft, keyright, keyup, keydown, keya, keyb;
#define KEY_LEFT keyleft
#define KEY_RIGHT keyright
#define KEY_UP keyup
#define KEY_DOWN keydown
#define KEY_A keya
#define KEY_B keyb
#else
//////////////////////////////////////////////////////////////////////
// AVR build
//////////////////////////////////////////////////////////////////////
#include <string.h>
#include <stdio.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#define ROMDATA(x) pgm_read_byte(&x)
#define ROMDATA16(x) pgm_read_word(&x)
#define KEY_LEFT !(PIND & 0x08)
#define KEY_RIGHT !(PIND & 0x04)
#define KEY_UP !(PIND & 0x80)
#define KEY_DOWN !(PIND & 0x01)
#define KEY_A !(PIND & 0x20)
#define KEY_B !(PIND & 0x10)
#endif

///////////////////////////////////////////////////////////////////////////////
// CONFIG
///////////////////////////////////////////////////////////////////////////////

#define INCLUDE_8X8_FONT
#define INCLUDE_4X4_FONT

///////////////////////////////////////////////////////////////////////////////
// MACROS
///////////////////////////////////////////////////////////////////////////////

// Millisecond delay. Mostly for hardware use. Does not work in SDL.
#define DELAY_MS(a) { unsigned short dt = tick + a; while (tick != dt) {} }

///////////////////////////////////////////////////////////////////////////////
// CONST DATA
///////////////////////////////////////////////////////////////////////////////

extern const unsigned char PROGMEM sintable[];

#ifdef INCLUDE_4X4_FONT
extern const unsigned char PROGMEM fontdata4[];
#endif

#ifdef INCLUDE_8X8_FONT
extern const unsigned char PROGMEM fontdata8[];
#endif

///////////////////////////////////////////////////////////////////////////////
// NONCONST DATA
///////////////////////////////////////////////////////////////////////////////

extern volatile unsigned short tick;
extern char tick_prescaler;
extern unsigned char fb[];

///////////////////////////////////////////////////////////////////////////////
// GRAPHICS ROUTINES
///////////////////////////////////////////////////////////////////////////////

extern void drawsprite(int x, int y, const unsigned char * mask, const unsigned char * data);

extern void putpixel(int x, int y, int c);

#ifdef INCLUDE_8X8_FONT
extern void drawchar8(int x, int y, int c);
extern void drawtext8(int x, int y, char *text);
#endif

#ifdef INCLUDE_4X4_FONT
extern void drawchar4(int x, int y, int c);
extern void drawtext4(int x, int y, char *text);
#endif

extern void scanlineon(unsigned char x, unsigned char y1, unsigned char y2);
extern void scanlineoff(char x, char y1, char y2);

///////////////////////////////////////////////////////////////////////////////
// LCD DRIVER
///////////////////////////////////////////////////////////////////////////////

extern void lcd_dump();
extern void lcd_init();

///////////////////////////////////////////////////////////////////////////////
// MIDI 
///////////////////////////////////////////////////////////////////////////////
unsigned char midi_tick();

#ifdef __cplusplus
} 
#endif
