#include <math.h>
#include "gametoy.h"

// 16-bit incrementer values for all notes in 8 octaves at 8kHz (Hz / 8000 * 64k),
// tuned at 440HZ on middle-A, and note number 60 is C5. The highest couple
// octaves can probably be ditched because they're simply too high.
// Hz for a note can be calculated by multiplying (or dividing) the value of the
// previous note with the 12th root of 2.
// Usage: increment a 16-bit variable with inc on every tick, and use upper 8 bits
// as pointer to 256-byte waveform table. (or use as is for saw)

#define USE_7812HZ

static const int PROGMEM increment[8*12] = {
#ifndef USE_7812HZ
// 15.6KHz
145, 153, 163, 172, 183, 193, 205, 217, 230, 244, 258, 274,
290, 307, 326, 345, 366, 387, 411, 435, 461, 488, 517, 548,
581, 615, 652, 691, 732, 775, 822, 870, 922, 977, 1035, 1097,
1162, 1231, 1304, 1382, 1464, 1551, 1644, 1741, 1845, 1955, 
2071, 2194,2325, 2463, 2609, 2765, 2929, 3103, 3288, 3483, 
3690, 3910, 4142, 4389, 4650, 4926, 5219, 5530, 5859, 6207, 
6576, 6967, 7381, 7820, 8285, 8778, 9300, 9853, 10439, 11060, 
11718, 12414, 13153, 13935, 14763, 15641, 16571, 17557, 18601, 
19707, 20879, 22120, 23436, 24829, 26306, 27870, 29527, 31283, 
33143
#else
// 7.8KHz
290, 307, 326, 345, 366, 387, 411, 435, 461, 488, 517, 548, 
581, 615, 652, 691, 732, 775, 822, 871, 922, 977, 1035, 1097, 
1162, 1231, 1305, 1382, 1464, 1551, 1644, 1742, 1845, 1955, 2071, 2194, 
2325, 2463, 2610, 2765, 2929, 3103, 3288, 3484, 3691, 3910, 4143, 4389, 
4650, 4927, 5220, 5530, 5859, 6207, 6577, 6968, 7382, 7821, 8286, 8779, 
9301, 9854, 10440, 11061, 11718, 12415, 13154, 13936, 14764, 15642, 16573, 
17558, 18602, 19708, 20880, 22122, 23437, 24831, 26308, 27872, 29529, 31285, 
33146, 35117, 37205, 39417, 41761, 44244, 46875, 49662, 52616, 55744, 59059, 
62571, 0 /*66292*/
#endif
};



const unsigned char PROGMEM song[] = 
{

#include "tune.h"

0x00,0x00,0x00,0x00};

unsigned short currentnote = 0;
#define CHANNELS 4
unsigned short osc[CHANNELS];
unsigned short adder[CHANNELS];
short volume[CHANNELS];
unsigned char nextvoice = 0;
unsigned short nextnote = 10;


unsigned char midi_tick()
{
	unsigned short d;
	unsigned char i;

	nextnote--;

	for (i = 0; i < CHANNELS; i++)
	{
		// Oscillate oscillators..
		osc[i] += adder[i];
		
		// and apply volume ramp
		volume[i]--;
		
		// don't let volume roll around
		if (volume[i] == 0)
			volume[i] = 1;
		
	}

	// If it's time for next note in the song data..
	while (nextnote == 0)
	{		
		// Fetch note
		unsigned char note = ROMDATA(song[currentnote + 2]);
		
		// Fetch oscilaltor incrementor based on note
		adder[nextvoice] = ROMDATA16(increment[note]);
		
		// Fetch starting volume
		volume[nextvoice] = ((unsigned short)ROMDATA(song[currentnote + 3])) << 8;

		// Fetch ticks until next note
		nextnote = ROMDATA(song[currentnote + 0]) + (ROMDATA(song[currentnote + 1]) << 8);

		// Move to the next note
		currentnote+=4;

		// Loop marker
		if (volume[nextvoice] == 0)
			currentnote = 0;

		// Set next active voice (round robin)
		nextvoice++;
		nextvoice &= CHANNELS-1;
	}

	d = 0;

	for (i = 0; i < CHANNELS; i++)
	{
		signed char data;
		
		// Retrieve sample
		data = ROMDATA(sintable[osc[i] >> 8])^0x80; // sine
		//data = (osc[i] >> 8); // saw

		// Apply effects
		//data = (data * (osc[i] >> 8)) >> 8; // ring mod
		
		// Apply volume
		data = (data * (volume[i] >> 8)) >> 8;
		
		// Accumulate
		d += data;
	}

	// Scale values back to 8 bit
	d>>=1;
	
	return (unsigned char)(d & 0xff);
}
