#include "gametoy.h"
#include "SDL.h"

#define PIXELON 0x2C342C
#define PIXELOFF 0x5D6436

SDL_Surface *screen;
SDL_AudioSpec *gAudioSpec = NULL;

extern "C" void mainloop();

int blend(int a, int b)
{
	return (((((a >>  0) & 0xff) + ((b >>  0) & 0xff) * 7) / 8) <<  0) |
	  	   (((((a >>  8) & 0xff) + ((b >>  8) & 0xff) * 7) / 8) <<  8) |
		   (((((a >> 16) & 0xff) + ((b >> 16) & 0xff) * 7) / 8) << 16);
}

void render()
{   
	static int lasttick = 0;

	if (SDL_GetTicks() - lasttick < 20)
		return;
	lasttick = SDL_GetTicks();
	tick = lasttick;

	// Lock surface if needed
	if (SDL_MUSTLOCK(screen))
		if (SDL_LockSurface(screen) < 0) 
			return;

	mainloop();

    ////////////////////////////////////////////////////////////////////
	// blit out
	
	int i,j;
	
	for (i = 0; i < 128; i++)
	{
		for (j = 0; j < 64; j++)
		{
			int c = (fb[i*8+j/8] >> (j & 7)) & 1;

			if (c)
				c = PIXELON;
			else
				c = PIXELOFF;

			c = blend(c, ((unsigned int*)screen->pixels)[(i*2+0) + (j*2+0) * (screen->pitch / 4)]);

			((unsigned int*)screen->pixels)[(i*2+0) + (j*2+0) * (screen->pitch / 4)] = c;
			((unsigned int*)screen->pixels)[(i*2+1) + (j*2+0) * (screen->pitch / 4)] = c;
			((unsigned int*)screen->pixels)[(i*2+0) + (j*2+1) * (screen->pitch / 4)] = c;
			((unsigned int*)screen->pixels)[(i*2+1) + (j*2+1) * (screen->pitch / 4)] = c;
		}
	}


	// Unlock if needed
    if (SDL_MUSTLOCK(screen)) 
        SDL_UnlockSurface(screen);

	// Tell SDL to update the whole screen
    SDL_UpdateRect(screen, 0, 0, 128*2, 64*2);    
}


void audiomixer(void *userdata, Uint8 *stream, int len)
{
	// Can't be used as a timer the same way as AVR hardware, since
	// SDL requests a block of audio at once..
	int i;
	for (i = 0; i < len; i++)
	{
		stream[i] = midi_tick();
	}
}

// Entry point
int main(int argc, char *argv[])
{
	keyleft = 0;
	keyright = 0;
	keyup = 0;
	keydown = 0;
	keya = 0;
	keyb = 0;

	// Initialize SDL's subsystems - in this case, only video.
    if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) 
	{
        fprintf(stderr, "Unable to init SDL: %s\n", SDL_GetError());
        exit(1);
    }

	// Register SDL_Quit to be called at exit; makes sure things are
	// cleaned up when we quit.
    atexit(SDL_Quit);
    
	// Attempt to create a 640x480 window with 32bit pixels.
    screen = SDL_SetVideoMode(128*2, 64*2, 32, SDL_SWSURFACE);
	
	// If we fail, return error.
    if ( screen == NULL ) 
	{
        fprintf(stderr, "Unable to set up video: %s\n", SDL_GetError());
        exit(1);
    }

    SDL_AudioSpec *as = new SDL_AudioSpec;
    as->freq = 7812;//15625;
    as->format = AUDIO_S8;
    as->channels = 1;
    as->samples = 4096;
    as->callback = audiomixer;
    as->userdata = NULL;
    if (SDL_OpenAudio(as, NULL) < 0)
    {
        fprintf(stderr, "Unable to init SDL audio: %s\n", SDL_GetError());
        exit(1);
    }
    gAudioSpec = as;
	SDL_PauseAudio(0); // let it play

	// Main loop: loop forever.
    while (1)
    {
		// Render stuff
        render();

		// Poll for events, and handle the ones we care about.
        SDL_Event event;
        while (SDL_PollEvent(&event)) 
        {
            switch (event.type) 
            {
            case SDL_KEYDOWN:
				switch (event.key.keysym.sym)
				{
				case SDLK_UP:
					keyup = 1;
					break;
				case SDLK_DOWN:
					keydown = 1;
					break;
				case SDLK_LEFT:
					keyleft = 1;
					break;
				case SDLK_RIGHT:
					keyright = 1;
					break;
				case SDLK_a:
					keya = 1;
					break;
				case SDLK_z:
					keyb = 1;
					break;
				}
                break;
            case SDL_KEYUP:
				// If escape is pressed, return (and thus, quit)
                if (event.key.keysym.sym == SDLK_ESCAPE)
                    return 0;
				switch (event.key.keysym.sym)
				{
				case SDLK_UP:
					keyup = 0;
					break;
				case SDLK_DOWN:
					keydown = 0;
					break;
				case SDLK_LEFT:
					keyleft = 0;
					break;
				case SDLK_RIGHT:
					keyright = 0;
					break;
				case SDLK_a:
					keya = 0;
					break;
				case SDLK_z:
					keyb = 0;
					break;
				}
                break;
            case SDL_QUIT:
                return(0);
            }
        }
    }
    return 0;
}

