#include "gametoy.h"

#ifndef _MSC_VER

// Hardware specific stuff. On desktop, SDL handles all of these.

///////////////////////////////////////////////////////////////////////////////
// TIMERS
///////////////////////////////////////////////////////////////////////////////


SIGNAL (SIG_OUTPUT_COMPARE1A)
{
	
	tick_prescaler++;
	if (tick_prescaler >= 8)
	{
		// ~1kHz tick counter
		tick_prescaler = 0;
		tick++;
	}
	
	// Fetch next sample from laura
	unsigned char d = midi_tick()^0x80;
	// Set the fast PWM channel trigger value
	OCR0 = d;	
}


///////////////////////////////////////////////////////////////////////////////
// LCD DRIVER
///////////////////////////////////////////////////////////////////////////////

enum LCD_BITS
{
	LCD_CS1    = 0x01,
	LCD_CS2    = 0x02,
	LCD_RST    = 0x04,
	LCD_RW     = 0x08,
	LCD_DI     = 0x10,
	LCD_ENABLE = 0x20
};

#define LCD_DATAPORT PORTA
#define LCD_DATAPORT_DIR DDRA
#define LCD_DATAPORT_IN PINA

#define LCD_CONTROLPORT PORTC
#define LCD_CONTROLPORT_DIR DDRC
#define LCD_CONTROLPORT_MASK 0x3f

// crossed wires, or bad data sheet?
//#define LCD_INSTRUCTION_READ LCD_CONTROLPORT = (LCD_CONTROLPORT & ~LCD_DI) | LCD_RW; // instruction / read
//#define LCD_DATA_WRITE       LCD_CONTROLPORT = (LCD_CONTROLPORT & ~LCD_RW) | LCD_DI; // data / write

#define LCD_DATA_WRITE       LCD_CONTROLPORT = (LCD_CONTROLPORT & ~LCD_DI) | LCD_RW; // instruction / read
#define LCD_INSTRUCTION_READ LCD_CONTROLPORT = (LCD_CONTROLPORT & ~LCD_RW) | LCD_DI; // data / write

#define LCD_INSTRUCTION_WRITE LCD_CONTROLPORT &= ~(LCD_RW | LCD_DI); // instruction / write

// Strobe 'enable' bit - i.e. clock the LCD controller
void e_strobe()
{
	LCD_CONTROLPORT |= LCD_ENABLE;
	LCD_CONTROLPORT &= ~LCD_ENABLE;
}

// TODO: the timings in this function could possibly be trimmed
// -> major speed optimization possibilities
void wait_lcd()
{
	unsigned short fallout = 100;
	LCD_DATAPORT_DIR = 0; // a port to read
	LCD_DATAPORT = 0xff;  // force signal high to be readable
	LCD_INSTRUCTION_READ;

	// try 100 times
	while (fallout)
	{
		fallout--;
		e_strobe();

	    LCD_CONTROLPORT |= LCD_ENABLE;
	    _delay_loop_2(5);
		
		unsigned char d = LCD_DATAPORT_IN;
	    LCD_CONTROLPORT &= ~LCD_ENABLE;

//		PORTB = d; // debug: show values as LEDs on the STK500 port B
		if ((d & 0x80) == 0)
		{
			// port back to write
			LCD_DATAPORT_DIR = 0xff;
			return;
		}
	}	
	// port back to write
	LCD_DATAPORT_DIR = 0xff;

	// If we get here, something is wrong.

	//DELAY_MS(10);	
}

// Output data to LCD
void write_lcd(unsigned char data)
{
	wait_lcd();
	LCD_DATA_WRITE;
	LCD_DATAPORT = data;
	e_strobe();
}

// Write a command to the LCD
void write_command(unsigned char command)
{
	wait_lcd();
	LCD_INSTRUCTION_WRITE;
	LCD_DATAPORT = command;
	e_strobe();
}

// Set active output page on the LCD
void set_page(unsigned char page)
{
	write_command(0xb8 + page);
}

// Set current address on the LCD
void set_address(unsigned char address)
{
	write_command(0x40 + address);
}

// Talk to controller 1
void set_cs1()
{
	LCD_CONTROLPORT = (LCD_CONTROLPORT & ~LCD_CS2) | LCD_CS1;
}

// Talk to controller 2
void set_cs2()
{
	LCD_CONTROLPORT = (LCD_CONTROLPORT & ~LCD_CS1) | LCD_CS2;
}

// Set up the LCD
// Timings in this function could be trimmed, but there's
// little sense in that as long as the function works..
void lcd_init()
{
	LCD_DATAPORT = 0;        // port A low
	LCD_DATAPORT_DIR = 0xff; // port A all outputs
	LCD_CONTROLPORT = 0;     // same for port C
	LCD_CONTROLPORT_DIR |= LCD_CONTROLPORT_MASK; // set control port to output

	LCD_INSTRUCTION_WRITE; // setup instruction writing mode
	LCD_CONTROLPORT |= LCD_CS1 | LCD_CS2; // talk to both controllers at once
		
	DELAY_MS(10);

	LCD_DATAPORT = 0xc0; // display start line = 0 (0-63)
	e_strobe();
	
	DELAY_MS(10);
	LCD_DATAPORT = 0x40; // set address = 0 (0-63)
	e_strobe();

	DELAY_MS(10);
	LCD_DATAPORT = 0xb8; // set page = 0 (0-7)
	e_strobe();

	DELAY_MS(10);
	LCD_DATAPORT = 0x3f; // display ON
	e_strobe();

}


// Dump framebuffer to LCD
void lcd_dump()
{	
	unsigned char a, b;

	for (a = 0; a < 8; a++)
	{
		set_cs1();
		set_page(a);
		set_address(0);
		for (b = 0; b < 64; b++)
			write_lcd(fb[a+b*8]);
		
		set_cs2();
		set_page(a);
		set_address(0);
		for (b = 0; b < 64; b++)
			write_lcd(fb[a+b*8+512]);
	}
}

#endif // not on desktop builds
