#include "gametoy.h"


///////////////////////////////////////////////////////////////////////////////
// GRAPHICS ROUTINES
///////////////////////////////////////////////////////////////////////////////

// Draw a sprite. Sprite data must be in ROM.
void drawsprite(int x, int y, const unsigned char * mask, const unsigned char * data)
{	
	int shift = 7 - (y & 7);	
	int offset;
	int x1 = 0;
	int x2 = 8;
	int i;

	if (y < -7 || y >= 63)
		return;

	if (x < 0)
	{
		if (x < -7)
			return;
		x1 = -x;
		x = 0;
	}
		
	if (x > 120)
	{
		if (x > 127)
			return;
		x2 = 127-x;
	}

	offset = (y >> 3) + x * 8;
	
	if (shift == 0)
	{				
		offset++;
		for (i = x1; i < x2; i++)
		{
			fb[offset] = fb[offset] & ~ROMDATA(mask[i]);
			fb[offset] = fb[offset] | ROMDATA(data[i]);
			offset += 8;
		}
	}
	else
	{		
		int oo = offset;
		offset+=1;
		
		if (y <= 56)
		{
			for (i = x1; i < x2; i++)
			{
				unsigned char maska = ROMDATA(mask[i]);
				unsigned char dataa = ROMDATA(data[i]);
				maska >>= shift;
				dataa >>= shift;

				fb[offset] = fb[offset] & ~maska;
				fb[offset] = fb[offset] | dataa;
				offset += 8;
			}
		}
		offset = oo;
		if (y >= 0)
		{
			for (i = x1; i < x2; i++)
			{
				unsigned char  maskb = ROMDATA(mask[i]);
				unsigned char  datab = ROMDATA(data[i]);
				maskb <<= 8-shift;
				datab <<= 8-shift;

				fb[offset] = fb[offset] & ~maskb;
				fb[offset] = fb[offset] | datab;
				offset += 8;
			}
		}
	}

}

// Putpixel - mostly for debugging
void putpixel(int x, int y, int c)
{
	if (x < 0 || x > 127 || y < 0 || y > 64)
	{
		return;
	}
	else
	{
		int b = 1 << (y & 7);
		y /= 8;
		if (c)
			fb[x * 8 + y] |= b;
		else
			fb[x * 8 + y] &= ~b;
	}
}

#ifdef INCLUDE_8X8_FONT

void drawchar8(int x, int y, int c)
{
	int i;
	for (i = 0; i < 8; i++)	
		fb[y+(x+i)*8] |= ROMDATA(fontdata8[c*8+i]);
}

void drawtext8(int x, int y, char *text)
{
	while (*text)
	{
		drawchar8(x,y,*text-32);
		text++;
		x+=8;
	}
}
#endif

#ifdef INCLUDE_4X4_FONT
void drawchar4(int x, int y, int c)
{
	int i;
	if (c & 1)
	{
		c &= 0xfe;
		for (i = 0; i < 4; i++)
			fb[y+((x+i)<<3)] |= ROMDATA(fontdata4[c*2+i]) & 0xf0;
	}
	else
	{
		for (i = 0; i < 4; i++)
			fb[y+((x+i)<<3)] |= ((ROMDATA(fontdata4[c*2+i]) & 0x0f) << 4);
	}
}

void drawtext4(int x, int y, char *text)
{
	while (*text)
	{
		drawchar4(x,y,*text-32);
		text++;
		x+=5;
	}
}
#endif

unsigned char foomask3(char y2, char d)
{
	unsigned char db = (1 << d) - 1;
	return db;
}

unsigned char foomask2(char y2, char d)
{
	char s;
	s = y2 & 7;
	return s;
}

unsigned char foomask1(char y2, char d)
{
	char s;
	s = y2 & 7;
	s = d - s;
	return s;
}

unsigned char foomask(char y2, char d)
{
	unsigned char db = (1 << d) - 1;
	char s;
	s = y2 & 7;
	s = d - s;
	db = db >> (s & 7);
	return db;
}

// Draw a full-on scanline.
void scanlineon(unsigned char x, unsigned char y1, unsigned char y2)
{
	unsigned char i;
	unsigned char d = y2 - y1;
	
	if (d < 8)
	{
		// special handling required
		unsigned char db = (1 << d) - 1;
		fb[x*8+y1/8] |= db << (y1 & 7);
		fb[x*8+y2/8] |= db >> ((d - (y2 & 7)) & 7);
		return;
	}

	fb[x*8+y1/8] |= 0xff << (y1 & 7);
	fb[x*8+y2/8] |= 0xff >> (7 - (y2 & 7));
	
	for (i = (y1/8)+1; i < (y2/8); i++)
	{
		fb[x*8+i] = 0xff;
	}
}

// Draw a full-off scanline.
void scanlineoff(char x, char y1, char y2)
{
	char i;
	unsigned char d = y2 - y1;

	if (d < 8)
	{
		// special handling required
		unsigned char db = (1 << d) - 1;
		fb[x*8+y1/8] &= ~(db << (y1 & 7));
		fb[x*8+y2/8] &= ~(db >> ((d - (y2 & 7)) & 7));
		return;
	}

	fb[x*8+y1/8] &= 0xff >> (7 - (y1 & 7));
	fb[x*8+y2/8] &= 0xff << ((y2 & 7));
	for (i = (y1/8)+1; i < (y2/8); i++)
	{
		fb[x*8+i] = 0;
	}
}
